//Kolmogorov - Smirnov test for continuous data
function pv=ks_test_cont(F)
  //pv ... p-value
  //F ... vector of values of distribution function in measured values
  //E.g. for  X=[3,6,8] and test of normality with N(5,4), 
  //       F=distfun_normcdf(X,5,sqrt(4));
  if argn(2)==1 then
    alpha=0.05;
  end

  [m,n]=size(F);
  N=m*n;
  F=matrix(F,1,N);    
  F1=gsort(F,'c','i');
  F2=(1:N)/N;
  DF1=max(F2-F1);
  DF2=max(F1(2:N)-F2(1:N-1));
  D=max(DF1,DF2);
  
  Stop=%F;
  j=1;
  Sum=0;
  while ~Stop 
    Addition=(-1)^(j-1)*exp(-2*j*j*N*D*D);
    Sum=Sum+Addition;
    if Sum==0 then
        Stop=%T;
    else
        Stop=(abs(Addition/Sum)<0.0000000001);
    end;
    j=j+1;
  end;
  pv=2*Sum;
endfunction



//Kolmogorov - Smirnov test for discrete data
function pv=ks_test_disc(F,Fminus1)
  //pv ... p-value
  //F ... vector of values of distribution function in measured values
  //E.g. for  X=[3,6,8] and test of normality with N(5,4), 
  //       F=distfun_normcdf(X,5,sqrt(4));
  //Fminus1 ... vector of values of distribution function in measured values 
  //minus one.
  //E.g. for  X=[3,6,8] and test of Poisson distribution Po(10), 
  //then Fminus1 is constructed as follows
  //
  //XX=X-1;
  //for i=1:100
  //   if XX(i)>=0
  //      Fminus1(1,i)=distfun_poisscdf(XX(i),10);
  //   else
  //      Fminus1(1,i)=0;
  //   end;
  //end;

  [m,n]=size(F);
  N=m*n;

  F1=matrix(F,1,N);
  F2=matrix(Fminus1,1,N);

  Krok=(1+sqrt(5))/4/%pi;
  E=(1:N).*(N+1:2*N)*Krok;
  e=E-fix(E);

  F4=F2+(F1-F2).*e;

  pv=ks_test_cont(F4);
endfunction

//Kolmogorov - Smirnov test for comparison of two samples
function pv=ks_test_2(X1,X2)
  //pv ... p-value
  //X1, X2 ... first and second sample
  if argn(2)==2 then
    alpha=0.05;
  end

  [m,n]=size(X1);
  N1=m*n;
  X1=matrix(X1,1,N1);    
  X1=gsort(X1,'c','i');
  F1=(1:N1)/N1;
  Z1=zeros(1,N1);

  [m,n]=size(X2);
  N2=m*n;
  X2=matrix(X2,1,N2);    
  X2=gsort(X2,'c','i');        
  F2=(1:N2)/N2;
  Z2=zeros(1,N2);

  X=[X1,X2];
  F1=[F1,Z2];
  F2=[Z1,F2];

  [X,k]=gsort(X,'c','i');
  F1=F1(k);
  F2=F2(k);

  for i=N1+N2:-1:2
    if X(i)==X(i-1)
      X(i-1)=[];
      F1(i)=max(F1(i),F1(i-1));
      F1(i-1)=[];
      F2(i)=max(F2(i),F2(i-1));
      F2(i-1)=[];
    end;
  end;

  [m,n]=size(X);

  for i=1:n
    F1(i)=max(F1(1:i));
    F2(i)=max(F2(1:i));
  end;

  D=max(abs(F1-F2));
  
  Ne=N1*N2/(N1+N2);
  SqrtNe=sqrt(Ne);
  Arg=(SqrtNe+0.12+0.11/SqrtNe)*D;
  
  Stop=%F;
  j=1;
  Sum=0;
  while ~Stop 
    Addition=(-1)^(j-1)*exp(-2*j*j*Arg*Arg);
    Sum=Sum+Addition;
    if Sum==0 then
        Stop=%T;
    else
        Stop=(abs(Addition/Sum)<0.0000000001);
    end;
    j=j+1;
  end;
  pv=2*Sum;
endfunction
